unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
  Spin, LazSerial, math, DateUtils, Clipbrd;

type

  { TMain_Form }

  TMain_Form = class(TForm)
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bezel_Shape1: TShape;
    dBV_Value_Label: TLabel;
    Label1: TLabel;
    Label6: TLabel;
    Shape2: TShape;
    Shape3: TShape;
    Zero_Button: TButton;
    Impedance_ComboBox: TComboBox;
    dBm_Value_Label: TLabel;
    dBr_Value_Label: TLabel;
    Data_Saved_Label: TLabel;
    Calibrated_Label: TLabel;
    Save_Data_Button: TButton;
    Connect_Button: TButton;
    Data_Received_Label: TLabel;
    Serial_Port_Edit: TEdit;
    Exit_Button: TButton;
    Calibrate_Button: TButton;
    LazSerial1: TLazSerial;
    Raw_Rx_Data_Label: TLabel;
    Calc_Value_Label: TLabel;
    Shape1: TShape;
    Bezel_Shape: TShape;
    procedure Connect_ButtonClick(Sender: TObject);
    procedure Exit_ButtonClick(Sender: TObject);
    procedure Calibrate_ButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure Save_Data_ButtonClick(Sender: TObject);
    procedure Zero_ButtonClick(Sender: TObject);
  private

  public
    raw_ADC: Integer;
    range, cal_range : Integer;
    RxStr: AnsiString;
    low_ADC_raw_point, high_ADC_raw_point: Integer;
    low_display_point, high_display_point, m, b: Real;
    save_filename: String;
    save_data_accepted, Calibrated: Boolean;
    dBr_Zero_Value: Real;
    calc_val : Real;
    Data_Transfer_Serial_Port: String;
    procedure Read_RxString(Sender: TObject);
    procedure Save_Settings(Sender: TObject);


  end;

var
  Main_Form: TMain_Form;

implementation
uses
  Unit2, Unit3, Unit4;

{$R *.lfm}

{ TMain_Form }


procedure TMain_Form.Read_RxString(Sender: TObject);

var  comma_pos: Integer;

begin
     Repeat
           RxStr := Main_Form.LazSerial1.ReadData; //
     Until (RxStr <> '') ;
     Main_Form.Raw_Rx_Data_Label.Caption := 'ADC,RANGE: ' + RxStr;
//     Write(RxStr);
     comma_pos :=  pos(',',RxStr);
     Main_Form.raw_ADC := StrToInt(copy(RxStr,1,comma_pos - 1));
     Main_Form.range := StrToInt(copy(RxStr,comma_pos + 1,1));
     if Main_Form.Data_Received_Label.Font.Color = $00444400 then Main_Form.Data_Received_Label.Font.Color := $00929200
     else Main_Form.Data_Received_Label.Font.Color := $00444400;
end;

procedure TMain_Form.Save_Settings(Sender: TObject);
var
        settings_File: TextFile ;
begin
//writeln('Save_Settings');

     if Save_Data_Form.Data_Transfer_LazSerial.Active then Save_Data_Form.Data_Transfer_LazSerial.close;
        if Main_Form.LazSerial1.Active then Main_Form.LazSerial1.close;
        AssignFile(settings_File,'saved_settings.cfg');
        Try
              Rewrite(settings_File);
              Writeln(settings_File,FloatToStr(Main_Form.m));
              Writeln(settings_File,FloatToStr(Main_Form.b));
              Writeln(settings_File,IntToStr(Main_Form.cal_range));
              Writeln(settings_File,Main_Form.Serial_Port_Edit.Text);
              Writeln(settings_File,Save_Data_Form.Data_Transfer_Serial_Port_Edit.Text);
              Writeln(settings_File,IntToStr(Main_Form.Top));
              Writeln(settings_File,IntToStr(Main_Form.Left));
              CloseFile(settings_File);
        Except
                 on E: EInOutError do
                 begin
                      ShowMessage('Error! Software cannot save its settings. Will revert to default values at next launch. Exiting...');
                      Sleep(2000);
                 end;
        end;
end;


procedure TMain_Form.FormCreate(Sender: TObject);

var
  tempString: String;
  settings_File: TextFile ;

begin
     {$IFDEF WINDOWS}
             Calc_Value_Label.Font.Name := 'Lucida Console';           //
             dBV_Value_Label.Font.Name := 'Lucida Console';
             dBm_Value_Label.Font.Name := 'Lucida Console';
             dBr_Value_Label.Font.Name := 'Lucida Console';
             Calc_Value_Label.Top := Calc_Value_Label.Top + 10;
             dBV_Value_Label.Top := dBV_Value_Label.Top + 10;
             dBm_Value_Label.Top := dBm_Value_Label.Top + 10;
             dBr_Value_Label.Top := dBr_Value_Label.Top + 10;
             Serial_Port_Edit.Top := Serial_Port_Edit.Top + 8;
             Impedance_ComboBox.Top := Impedance_ComboBox.Top + 8;
     {$ENDIF}
     save_data_accepted := False;
     save_filename := './Results.txt';
     dBr_Zero_Value := 0;
     Main_Form.Show;
     Main_Form.Visible := True;
     Splash_Form.Show;
     Sleep(2000);
     Splash_Form.Hide;

     // Retrieving the previously-saved parameters from the text file
     AssignFile(settings_File,'saved_settings.cfg');
     Try
        Reset(settings_File);
        Readln(settings_File,tempString);
        m := StrToFloat(tempString);
        Readln(settings_File,tempString);
        b := StrToFloat(tempString);
        Readln(settings_File,tempString);
        cal_range := StrToInt(tempString);
        Readln(settings_File,tempString);
        Serial_Port_Edit.Text := tempString;
        Readln(settings_File,tempString);
        Data_Transfer_Serial_Port := tempString;
        Readln(settings_File,tempString);
        Main_Form.Top := StrToInt(tempString);
        Readln(settings_File,tempString);
        Main_Form.Left := StrToInt(tempString);
        CloseFile(settings_File);
     Except
        on E: EInOutError do// missing or inaccessible settings file
        begin
             ShowMessage('Error! Software cannot retrieve the previous settings and calibration values. Please perform calibration.');
             m := 0;
             b := 0;
             cal_range := 0;
        end;
     end;
     if m = 0 then
     begin
          Calibrated_label.Font.Color := $00444400;
          Calibrated := False;
     end
     else begin
          Calibrated_label.Font.Color := $00929200;
          Calibrated := True;
     end;

     Application.ProcessMessages;          // Refreshes the screen

end;

procedure TMain_Form.Save_Data_ButtonClick(Sender: TObject);
begin
     Main_Form.Enabled := False;
     Save_Data_Form.show;
end;

procedure TMain_Form.Zero_ButtonClick(Sender: TObject);
begin
     dBr_Zero_Value := 10 * log10(Power(calc_val,2) / StrToInt(Impedance_ComboBox.Text) / 0.001);
end;

procedure TMain_Form.Connect_ButtonClick(Sender: TObject);

var
   data_file: TextFile ;
   ThenSecs : TDateTime;
   TempStr : String;

begin
       // Disable the various components during configuration transfer to Arduino.
     LazSerial1.Device := Serial_Port_Edit.Text;
     ThenSecs := 0;
     try
        LazSerial1.open;
     except
           On E :Exception do
           begin
               ShowMessage('Error! Serial port is unavailable or not present. Verify port naming.');
                exit;
           end;
     end;
     Main_Form.Calc_Value_Label.Font.Color := $00929200;
     Raw_Rx_Data_Label.Font.Color := $00929200;
     dBV_Value_Label.Font.Color := $00929200;
     dBm_Value_Label.Font.Color := $00929200;
     dBr_Value_Label.Font.Color := $00929200;
     Calibrate_Button.Enabled := True;
     Serial_Port_Edit.Enabled := False;
     Connect_Button.Enabled := False;
     repeat
           Sleep(10);
           Data_Saved_Label.Font.Color := $00444400;
           Read_RxString(self);
           if (Calibrated = True) then calc_val  := ((m * raw_ADC) + b) * Power(10, 2 - ((2 + cal_range) mod 3)) / Power(10, 6 - range)
           else calc_val  :=  raw_ADC / 32767 * 2.048 / Power(10, 4 - range);
           Calc_Value_Label.Caption := FloatToStrF(calc_val,ffFixed, 5, 8 - range) + ' V';
           dBV_Value_Label.Caption := FloatToStrF(20 * log10(calc_val),ffFixed,4,2);
           dBm_Value_Label.Caption := FloatToStrF(10 * log10(Power(calc_val,2) / StrToInt(Impedance_ComboBox.Text) / 0.001),ffFixed,4,2);
           dBr_Value_Label.Caption := FloatToStrF(StrToFloat(dBm_Value_Label.Caption) - dBr_Zero_Value,ffFixed,4,2);
           dBm_Value_Label.Caption := dBm_Value_Label.Caption + ' dBm';
           if copy(dBV_Value_Label.Caption,1,1) <> '-' then dBV_Value_Label.Caption := '+' + dBV_Value_Label.Caption;
           dBV_Value_Label.Caption := dBV_Value_Label.Caption + ' dBV';
           if copy(dBm_Value_Label.Caption,1,1) <> '-' then dBm_Value_Label.Caption := '+' + dBm_Value_Label.Caption;
           dBr_Value_Label.Caption := dBr_Value_Label.Caption + ' dBr';
           if copy(dBr_Value_Label.Caption,1,1) <> '-' then dBr_Value_Label.Caption := '+' + dBr_Value_Label.Caption;
           Application.ProcessMessages;          // Refreshes the screen

           TempStr := Copy(Calc_Value_Label.Caption,1, Length(Calc_Value_Label.Caption) - 2) + ',' +
           Copy(dBV_Value_Label.Caption,1, Length(dBV_Value_Label.Caption) - 4) + ',' +
           Copy(dBm_Value_Label.Caption,1, Length(dBm_Value_Label.Caption) - 4) + ',' +
           Copy(dBr_Value_Label.Caption,1, Length(dBr_Value_Label.Caption) - 4) + ',' +
           IntToStr(raw_ADC) + ',' +
           IntToStr(range) + ',' +
           DateTimeToStr(Now);
           // Saving data to file
           if Save_Data_Form.Save_Data_CheckBox.Checked and save_data_accepted and (MilliSecondsBetween(Now,ThenSecs) >= (Save_Data_Form.Saving_Time_SpinEdit.Value * 1000)) then
           begin
                ThenSecs := Now;
                Try
                   AssignFile(data_file,save_filename);
                   if FileExists(save_filename) then Append(data_file) else Rewrite(data_file);
                   Writeln(data_file,TempStr);
//                   Clipboard.AsText := TempStr;
                   Data_Saved_Label.Font.Color := $00929200;
                   Application.ProcessMessages;          // Refreshes the screen
                   CloseFile(data_file);
                Except
                   on E: EInOutError do
                   begin
                         ShowMessage('Error! Cannot save data to file. Check disk access...');
                   end;
                end;
            end;
           // Sending data to Serial Port
           if Save_Data_Form.Send_Data_CheckBox.Checked and save_data_accepted and (Save_Data_Form.Data_Transfer_LazSerial.Active = True) then
           begin
                if Save_Data_Form.Data_Transfer_LazSerial.DataAvailable() and  (Save_Data_Form.Data_Transfer_LazSerial.ReadData() = 'T')
                then Save_Data_Form.Data_Transfer_LazSerial.WriteData(TempStr + #13#10);
           end;
           // Sending data to Clipboard
           if Save_Data_Form.Send_Data_to_Clipboard_CheckBox.Checked and save_data_accepted then Clipboard.AsText := TempStr + #13#10;
     until False;    // Repeats forever
end;

procedure TMain_Form.Exit_ButtonClick(Sender: TObject);

begin
//     writeln('Exit_ButtonClick');
     Save_Settings(Self);
     Halt;
end;

procedure TMain_Form.Calibrate_ButtonClick(Sender: TObject);
begin
     Main_Form.Enabled := False;
     Calibrate_Form.show;
end;

procedure TMain_Form.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
//     writeln('FormClose');
     Halt;
end;

procedure TMain_Form.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
//     writeln('FormCloseQuery');
     Save_Settings(Self);
end;


end.

